.my-apodWidget {
  display: flex;
  flex-direction: column;
  align-items: center;
  overflow: visible;
}
.dm_Widget-main *{
  overflow: visible;
}

#inhpc-datamanagement *
{
  overflow: auto;
}

#

#actionTollbar, #settingsWidget, #infoWidget
{
  height: 200px;
}